# Matthew Dukes
# Final Project - Digital Ruler

import board, neopixel, time, digitalio
from adafruit_debouncer import Button
from analogio import AnalogIn
import os, ssl, wifi, socketpool
import adafruit_minimqtt.adafruit_minimqtt as MQTT

aio_username = os.getenv("AIO_USERNAME")
aio_key = os.getenv("AIO_KEY")

cm_feed = aio_username + "/feeds/cm_feed"
in_feed = aio_username + "/feeds/in_feed"

def connected(client, userdata, flags, rc):
    print("Connected to IO")
    client.subscribe(cm_feed)
    client.subscribe(in_feed)

def disconnected(client, userdata, rc):
    print("Disconnected from IO")

def message(client, topic, message):
#     print(f"topic: {topic}, message: {message}")
    message = float(message)
    if topic == cm_feed:
        if message <= 48 and message >=0:
            num_pixels = convertToPixels("cm", message)
            pixels_off = strip_leds-num_pixels
            strip[0: num_pixels] = colors[0] * num_pixels
            strip[num_pixels: strip_leds] = BLACK * pixels_off
            time.sleep(5)
        else:
            print("\nThis length does not fit on the ruler")
    elif topic == in_feed:
        if message <= 19 and message >=0:
            num_pixels = convertToPixels("in", message)
            pixels_off = strip_leds-num_pixels
            strip[0: num_pixels] = colors[2] * num_pixels
            strip[num_pixels: strip_leds] = BLACK * pixels_off
            time.sleep(5)
        else:
            print("\nThis length does not fit on the ruler")

print(f"Connecting to WIFI")
wifi.radio.connect(os.getenv("WIFI_SSID"), os.getenv("WIFI_PASSWORD"))
print("Connected!")

pool = socketpool.SocketPool(wifi.radio)

mqtt_client = MQTT.MQTT(
    broker=os.getenv("BROKER"),
    port=os.getenv("PORT"),
    username=aio_username,
    password=aio_key,
    socket_pool=pool,
    ssl_context=ssl.create_default_context()
)

mqtt_client.on_connect = connected
mqtt_client.on_disconnect = disconnected
mqtt_client.on_message = message

print("Connecting to Adafruit IO")
mqtt_client.connect()
# end adafruit io code0


# led strip
strip_pin = board.GP0
strip_leds = 30
strip = neopixel.NeoPixel(strip_pin, strip_leds)

# buttons
button_input = digitalio.DigitalInOut(board.GP15)
button_input.switch_to_input(digitalio.Pull.UP)
button_1 = Button(button_input)

button_input = digitalio.DigitalInOut(board.GP16)
button_input.switch_to_input(digitalio.Pull.UP)
button_2 = Button(button_input)

# potentiometer
potentiometer = AnalogIn(board.GP26) # check config

# color setup
RED = (255, 0, 0)
GREEN = (0, 255, 0)
BLUE = (0, 0, 255)
ORANGE = (255, 40, 0)
PURPLE = (180, 0, 255)
AQUA = (0, 255, 255)
BLACK = (0, 0, 0)

colors = [RED, ORANGE, BLUE, PURPLE, AQUA, GREEN]
units = ["centimeters", "meters", "inches", "feet", "leds", "Yodas"]
curr_index = 0


# math
led_length = 1.6
def convert(unit, pixels):
    if unit == 0: # cm
        return led_length * pixels
    elif unit == 1: # m
        return led_length * pixels * .01
    elif unit == 2: # in
        ret = led_length * pixels / 2.54
        return round(ret, 2)
    elif unit == 3: # ft
        ret = led_length * pixels / 30.48
        return round(ret, 2)
    elif unit == 4: # led
        return pixels
    elif unit == 5: # yoda
        ret = led_length * pixels / 40.64
        return round(ret, 2)

def convertToPixels(unit, length):
    if unit == "cm":
        return round(length/led_length)
    elif unit == "in":
        return round(length * 2.54 / led_length)

while True:
    mqtt_client.loop()
    value = potentiometer.value
    value = value / 65535
    num_pixels = int((value)*strip_leds)
    pixels_off = strip_leds-num_pixels
    strip[0: num_pixels] = colors[curr_index] * num_pixels
    strip[num_pixels: strip_leds] = BLACK * pixels_off
    button_1.update()
    button_2.update()
    if button_2.pressed:
        print("\n" + str(convert(curr_index, num_pixels)) + " " + units[curr_index])
        time.sleep(0.2)
    if button_1.pressed:
        if curr_index == 5:
            curr_index = 0
        else:
            curr_index += 1
